@@ -54,6 +54,7 @@ sip_code_t sip_codes[] = {
     { SIP_METHOD_NOTIFY,    "NOTIFY" },
     { SIP_METHOD_OPTIONS,   "OPTIONS" },
     { SIP_METHOD_PUBLISH,   "PUBLISH" },
+    { SIP_METHOD_KDMQ,      "KDMQ" },
     { SIP_METHOD_MESSAGE,   "MESSAGE" },
     { SIP_METHOD_CANCEL,    "CANCEL" },
     { SIP_METHOD_BYE,       "BYE" },
@@ -192,7 +193,7 @@ sip_init(int limit, int only_calls, int no_incomplete)
             "^(X-Call-ID|X-CID):[ ]*([^ ]+)[ ]*\r$", match_flags);
     }
     regcomp(&calls.reg_response, "^SIP/2.0[ ]*(([0-9]{3}) [^\r]*)[ ]*\r", match_flags & ~REG_NEWLINE);
-    regcomp(&calls.reg_cseq, "^CSeq:[ ]*([0-9]+) .+\r$", match_flags);
+    regcomp(&calls.reg_cseq, "^CSeq:[ ]*([0-9]{1,10}) .+\r$", match_flags);
     regcomp(&calls.reg_from, "^(From|f):[ ]*[^:]*:(([^@>]+)@?[^\r>;]+)", match_flags);
     regcomp(&calls.reg_to, "^(To|t):[ ]*[^:]*:(([^@>]+)@?[^\r>;]+)", match_flags);
     regcomp(&calls.reg_valid, "^([A-Z]+ [a-zA-Z]+:|SIP/2.0 [0-9]{3})", match_flags & ~REG_NEWLINE);
@@ -307,6 +308,11 @@ sip_validate_packet(packet_t *packet)
     }
 
     if (content_len < bodylen) {
+        // Check body ends with '\r\n'
+        if (payload[pmatch[1].rm_so + content_len - 1] != '\n')
+            return VALIDATE_NOT_SIP;
+        if (payload[pmatch[1].rm_so + content_len - 2] != '\r')
+            return VALIDATE_NOT_SIP;
         // We got more than one SIP message in the same packet
         packet_set_payload(packet, payload, pmatch[1].rm_so + content_len);
         return VALIDATE_MULTIPLE_SIP;
@@ -716,7 +722,7 @@ sip_parse_msg_media(sip_msg_t *msg, const u_char *payload)
 
         // Check if we have attribute format string
         if (!strncmp(line, "a=rtpmap:", 9)) {
-            if (media && sscanf(line, "a=rtpmap:%u %[^ ]", &media_fmt_code, media_format)) {
+            if (media && sscanf(line, "a=rtpmap:%u %30[^ ]", &media_fmt_code, media_format)) {
                 media_add_format(media, media_fmt_code, media_format);
             }
         }
@@ -819,7 +825,7 @@ sip_set_match_expression(const char *expr, int insensitive, int invert)
 #ifdef WITH_PCRE
     const char *re_err = NULL;
     int32_t err_offset;
-    int32_t pflags = PCRE_UNGREEDY;
+    int32_t pflags = PCRE_UNGREEDY | PCRE_DOTALL;
 
     if (insensitive)
         pflags |= PCRE_CASELESS;
@@ -922,7 +928,11 @@ sip_get_msg_header(sip_msg_t *msg, char *out)
     msg_get_attribute(msg, SIP_ATTR_DST, to_addr);
 
     // Get msg header
-    sprintf(out, "%s %s %s -> %s", date, time, from_addr, to_addr);
+    if (setting_enabled(SETTING_DISPLAY_ALIAS)) {
+        sprintf(out, "%s %s %s -> %s", date, time, get_alias_value(from_addr), get_alias_value(to_addr));
+    } else {
+        sprintf(out, "%s %s %s -> %s", date, time, from_addr, to_addr);
+    }
     return out;
 }
 
